@@ -26,30 +26,34 @@
 #' Built-in univariate imputation methods are:
 #'
 #' \tabular{lll}{
-#' \code{pmm}          \tab any     \tab Predictive mean matching\cr
-#' \code{midastouch}   \tab any     \tab Weighted predictive mean matching\cr
-#' \code{sample}       \tab any     \tab Random sample from observed values\cr
-#' \code{cart}         \tab any     \tab Classification and regression trees\cr
-#' \code{rf}           \tab any     \tab Random forest imputations\cr
-#' \code{mean}         \tab numeric \tab Unconditional mean imputation\cr
-#' \code{norm}         \tab numeric \tab Bayesian linear regression\cr
-#' \code{norm.nob}     \tab numeric \tab Linear regression ignoring model error\cr
-#' \code{norm.boot}    \tab numeric \tab Linear regression using bootstrap\cr
-#' \code{norm.predict} \tab numeric \tab Linear regression, predicted values\cr
-#' \code{quadratic}    \tab numeric \tab Imputation of quadratic terms\cr
-#' \code{ri}           \tab numeric \tab Random indicator for nonignorable data\cr
-#' \code{logreg}       \tab binary  \tab Logistic regression\cr
-#' \code{logreg.boot}  \tab binary  \tab Logistic regression with bootstrap\cr
-#' \code{polr}         \tab ordered \tab Proportional odds model\cr
-#' \code{polyreg}      \tab unordered\tab Polytomous logistic regression\cr
-#' \code{lda}          \tab unordered\tab Linear discriminant analysis\cr
-#' \code{2l.norm}      \tab numeric  \tab Level-1 normal heteroscedastic\cr
-#' \code{2l.lmer}      \tab numeric  \tab Level-1 normal homoscedastic, lmer\cr
-#' \code{2l.pan}       \tab numeric  \tab Level-1 normal homoscedastic, pan\cr
-#' \code{2l.bin}       \tab binary   \tab Level-1 logistic, glmer\cr
-#' \code{2lonly.mean}  \tab numeric  \tab Level-2 class mean\cr
-#' \code{2lonly.norm}  \tab numeric  \tab Level-2 class normal\cr
-#' \code{2lonly.pmm}   \tab any      \tab Level-2 class predictive mean matching
+#' \code{pmm}               \tab any     \tab Predictive mean matching\cr
+#' \code{midastouch}        \tab any     \tab Weighted predictive mean matching\cr
+#' \code{sample}            \tab any     \tab Random sample from observed values\cr
+#' \code{cart}              \tab any     \tab Classification and regression trees\cr
+#' \code{rf}                \tab any     \tab Random forest imputations\cr
+#' \code{mean}              \tab numeric \tab Unconditional mean imputation\cr
+#' \code{norm}              \tab numeric \tab Bayesian linear regression\cr
+#' \code{norm.nob}          \tab numeric \tab Linear regression ignoring model error\cr
+#' \code{norm.boot}         \tab numeric \tab Linear regression using bootstrap\cr
+#' \code{norm.predict}      \tab numeric \tab Linear regression, predicted values\cr
+#' \code{lasso.norm}        \tab numeric \tab Lasso linear regression\cr
+#' \code{lasso.select.norm} \tab numeric \tab Lasso select + linear regression\cr
+#' \code{quadratic}         \tab numeric \tab Imputation of quadratic terms\cr
+#' \code{ri}                \tab numeric \tab Random indicator for nonignorable data\cr
+#' \code{logreg}            \tab binary  \tab Logistic regression\cr
+#' \code{logreg.boot}       \tab binary  \tab Logistic regression with bootstrap\cr
+#' \code{lasso.logreg}      \tab binary  \tab Lasso logistic regression\cr
+#' \code{lasso.select.logreg}\tab binary  \tab Lasso select + logistic regression\cr
+#' \code{polr}              \tab ordered \tab Proportional odds model\cr
+#' \code{polyreg}           \tab unordered\tab Polytomous logistic regression\cr
+#' \code{lda}               \tab unordered\tab Linear discriminant analysis\cr
+#' \code{2l.norm}           \tab numeric  \tab Level-1 normal heteroscedastic\cr
+#' \code{2l.lmer}           \tab numeric  \tab Level-1 normal homoscedastic, lmer\cr
+#' \code{2l.pan}            \tab numeric  \tab Level-1 normal homoscedastic, pan\cr
+#' \code{2l.bin}            \tab binary   \tab Level-1 logistic, glmer\cr
+#' \code{2lonly.mean}       \tab numeric  \tab Level-2 class mean\cr
+#' \code{2lonly.norm}       \tab numeric  \tab Level-2 class normal\cr
+#' \code{2lonly.pmm}        \tab any      \tab Level-2 class predictive mean matching
 #' }
 #'
 #' These corresponding functions are coded in the \code{mice} library under
@@ -183,6 +187,11 @@
 #' One may also use one of the following keywords: \code{"arabic"}
 #' (right to left), \code{"monotone"} (ordered low to high proportion
 #' of missing data) and \code{"revmonotone"} (reverse of monotone).
+#' \emph{Special case}: If you specify both \code{visitSequence = "monotone"} and
+#' \code{maxit = 1}, then the procedure will edit the \code{predictorMatrix}
+#' to conform to the monotone pattern. Realize that convergence in one
+#' iteration is only guaranteed if the missing data pattern is actually
+#' monotone. The procedure does not check this.
 #' @param formulas A named list of formula's, or expressions that
 #' can be converted into formula's by \code{as.formula}. List elements
 #' correspond to blocks. The block to which the list element applies is
@@ -199,6 +208,7 @@
 #' executed within the \code{sampler()} function to post-process
 #' imputed values during the iterations.
 #' The default is a vector of empty strings, indicating no post-processing.
+#' Multivariate (block) imputation methods ignore the \code{post} parameter.
 #' @param defaultMethod A vector of length 4 containing the default
 #' imputation methods for 1) numeric data, 2) factor data with 2 levels, 3)
 #' factor data with > 2 unordered levels, and 4) factor data with > 2
@@ -212,7 +222,9 @@
 #' Use \code{print=FALSE} for silent computation.
 #' @param seed An integer that is used as argument by the \code{set.seed()} for
 #' offsetting the random number generator. Default is to leave the random number
-#' generator alone.
+#' generator alone. Versions later than 3.13.11 reset the random generator to the
+#' state before calling \code{mice()}. This effectively isolates the \code{mice}
+#' random generator from the calling environment.
 #' @param data.init A data frame of the same size and type as \code{data},
 #' without missing data, used to initialize imputations before the start of the
 #' iterative process.  The default \code{NULL} implies that starting imputation
@@ -234,7 +246,7 @@
 #' @references Van Buuren, S., Groothuis-Oudshoorn, K. (2011). \code{mice}:
 #' Multivariate Imputation by Chained Equations in \code{R}. \emph{Journal of
 #' Statistical Software}, \bold{45}(3), 1-67.
-#' \url{https://www.jstatsoft.org/v45/i03/}
+#' \doi{10.18637/jss.v045.i03}
 #'
 #' Van Buuren, S. (2018).
 #' \href{https://stefvanbuuren.name/fimd/sec-FCS.html#sec:MICE}{\emph{Flexible Imputation of Missing Data. Second Edition.}}
@@ -289,7 +301,13 @@ mice <- function(data,
                  ...) {
   call <- match.call()
   check.deprecated(...)
-  if (!is.na(seed)) set.seed(seed)
+
+  # set local seed, reset random state generator after function aborts
+  if (is.na(seed)) {
+    withr::local_preserve_seed()
+  } else {
+    withr::local_seed(seed)
+  }
 
   # check form of data and m
   data <- check.dataform(data)
@@ -367,9 +385,17 @@ mice <- function(data,
 
   chk <- check.cluster(data, predictorMatrix)
   where <- check.where(where, data, blocks)
+
+  # check visitSequence, edit predictorMatrix for monotone
+  user.visitSequence <- visitSequence
   visitSequence <- check.visitSequence(visitSequence,
-    data = data,
-    where = where, blocks = blocks
+    data = data, where = where, blocks = blocks
+  )
+  predictorMatrix <- edit.predictorMatrix(
+    predictorMatrix = predictorMatrix,
+    visitSequence = visitSequence,
+    user.visitSequence = user.visitSequence,
+    maxit = maxit
   )
   method <- check.method(
     method = method, data = data, where = where,
@@ -408,7 +434,7 @@ mice <- function(data,
   to <- from + maxit - 1
   q <- sampler(
     data, m, ignore, where, imp, blocks, method,
-    visitSequence,predictorMatrix, formulas, blots,
+    visitSequence, predictorMatrix, formulas, blots,
     post, c(from, to), printFlag, ...
   )
 
@@ -433,7 +459,8 @@ mice <- function(data,
     ignore = ignore,
     seed = seed,
     iteration = q$iteration,
-    lastSeedValue = .Random.seed,
+    lastSeedValue = get(".Random.seed", envir = globalenv(), mode = "integer",
+                        inherits = FALSE),
     chainMean = q$chainMean,
     chainVar = q$chainVar,
     loggedEvents = loggedEvents,
