@@ -11,7 +11,7 @@
 #' @inheritParams mice.impute.pmm
 #' @param ums A string containing the specification of the
 #' unidentifiable part of the imputation model (the *unidentifiable
-#' model specification”), that is, the desired \eqn{\delta}-adjustment
+#' model specification"), that is, the desired \eqn{\delta}-adjustment
 #' (offset) as a function of other variables and values for the
 #' corresponding deltas (sensitivity parameters). See details.
 #' @param umx An auxiliary data matrix containing variables that do
@@ -23,7 +23,7 @@
 #' @details
 #' This function imputes data that are thought to be Missing Not at
 #' Random (MNAR) by the NARFCS method. The NARFCS procedure
-#' (Leacy, 2016; Tompsett et al, 2018) generalises the so-called
+#' (Tompsett et al, 2018) generalises the so-called
 #' \eqn{\delta}-adjustment sensitivity analysis method of Van Buuren,
 #' Boshuizen & Knook (1999) to the case with multiple incomplete
 #' variables within the FCS framework. In practical terms, the
@@ -40,7 +40,8 @@
 #' at least one argument \code{ums} and, optionally, a second
 #' argument \code{umx}.
 #' For example, the high-level call might like something like
-#' \code{mice(nhanes[, c(2, 4)], method = c("pmm", "mnar.norm"), blots = list(chl = list(ums = "-3+2*bmi")))}.
+#' \code{mice(nhanes[, c(2, 4)], method = c("pmm", "mnar.norm"),
+#' blots = list(chl = list(ums = "-3+2*bmi")))}.
 #'
 #' The \code{ums} parameter is required, and might look like this:
 #' \code{"-4+1*Y"}. The \code{ums} specifcation must have the
@@ -91,10 +92,6 @@
 #'
 #' @author Margarita Moreno-Betancur, Stef van Buuren, Ian R. White, 2020.
 #' @references
-#' Leacy, F.P. (2016). \emph{Multiple imputation under missing not at
-#' random assumptions via fully conditional specification}.
-#' Dissertation, University of Cambridge, UK.
-#'
 #' Tompsett, D. M., Leacy, F., Moreno-Betancur, M., Heron, J., &
 #' White, I. R. (2018). On the use of the not-at-random fully
 #' conditional specification (NARFCS) procedure in practice.
@@ -160,5 +157,7 @@ mice.impute.mnar.norm <- function(y, ry, x, wy = NULL,
   parm <- .norm.draw(y, ry, x, ...)
 
   ## Draw imputations
-  x[wy, ] %*% parm$beta + u$x[wy, ] %*% u$delta + rnorm(sum(wy)) * parm$sigma
+  return(x[wy, , drop = FALSE] %*% parm$beta +
+    u$x[wy, , drop = FALSE] %*% u$delta +
+    rnorm(sum(wy)) * parm$sigma)
 }
