@@ -18,7 +18,7 @@
 #' @references van Buuren S and Groothuis-Oudshoorn K (2011). \code{mice}:
 #' Multivariate Imputation by Chained Equations in \code{R}. \emph{Journal of
 #' Statistical Software}, \bold{45}(3), 1-67.
-#' \url{https://www.jstatsoft.org/v45/i03/}
+#' \doi{10.18637/jss.v045.i03}
 #' @keywords multivariate
 #' @examples
 #' imp <- mice(nhanes2, m = 2, print = FALSE, seed = 14221)
@@ -43,8 +43,9 @@ with.mids <- function(data, expr, ...) {
   for (i in seq_along(analyses)) {
     data.i <- complete(data, i)
     analyses[[i]] <- eval(expr = substitute(expr), envir = data.i, enclos = parent.frame())
-    if (is.expression(analyses[[i]]))
+    if (is.expression(analyses[[i]])) {
       analyses[[i]] <- eval(expr = analyses[[i]], envir = data.i, enclos = parent.frame())
+    }
   }
 
   # return the complete data analyses as a list of length nimp
